@@ -1,5 +1,4 @@
 /* eslint-disable @typescript-eslint/no-var-requires */
-const path = require('path');
 const { merge } = require('webpack-merge');
 const common = require('./webpack.common.js');
 const { stylePaths } = require('./stylePaths');
@@ -29,6 +28,8 @@ module.exports = merge(common('development'), {
           '/cluster-api',
           '/inventory-api',
           '/inventory-payload-api',
+          '/must-gather-api',
+          '/get-certificate',
         ],
         target: `http://localhost:${EXPRESS_PORT}`,
       },
