@@ -4,15 +4,15 @@ import badgeStyles from '@patternfly/react-styles/css/components/Badge/badge';
 import formStyles from '@patternfly/react-styles/css/components/FormControl/form-control';
 import buttonStyles from '@patternfly/react-styles/css/components/Button/button';
 import { css } from '@patternfly/react-styles';
-import TimesCircleIcon from '@patternfly/react-icons/dist/js/icons/times-circle-icon';
-import CheckCircleIcon from '@patternfly/react-icons/dist/js/icons/check-circle-icon';
-import ExclamationTriangleIcon from '@patternfly/react-icons/dist/js/icons/exclamation-triangle-icon';
-import ExclamationCircleIcon from '@patternfly/react-icons/dist/js/icons/exclamation-circle-icon';
+import TimesCircleIcon from '@patternfly/react-icons/dist/esm/icons/times-circle-icon';
+import CheckCircleIcon from '@patternfly/react-icons/dist/esm/icons/check-circle-icon';
+import ExclamationTriangleIcon from '@patternfly/react-icons/dist/esm/icons/exclamation-triangle-icon';
+import ExclamationCircleIcon from '@patternfly/react-icons/dist/esm/icons/exclamation-circle-icon';
 import { SelectMenu } from './SelectMenu';
 import { SelectOption, SelectOptionObject } from './SelectOption';
 import { SelectGroup, SelectGroupProps } from './SelectGroup';
 import { SelectToggle } from './SelectToggle';
-import { SelectContext, SelectVariant, SelectDirection, KeyTypes } from './selectConstants';
+import { SelectContext, SelectVariant, SelectPosition, SelectDirection, KeyTypes } from './selectConstants';
 import { Chip, ChipGroup, ChipGroupProps } from '../ChipGroup';
 import { Spinner } from '../Spinner';
 import {
@@ -46,6 +46,8 @@ export interface SelectProps
   children?: React.ReactElement[];
   /** Classes applied to the root of the Select */
   className?: string;
+  /** Indicates where menu will be aligned horizontally */
+  position?: SelectPosition | 'right' | 'left';
   /** Flag specifying which direction the Select menu expands */
   direction?: 'up' | 'down';
   /** Flag to indicate if select is open */
@@ -58,6 +60,10 @@ export interface SelectProps
   isDisabled?: boolean;
   /** Flag to indicate if the typeahead select allows new items */
   isCreatable?: boolean;
+  /** Flag indicating if placeholder styles should be applied */
+  hasPlaceholderStyle?: boolean;
+  /** @beta Flag indicating if the creatable option should set its value as a SelectOptionObject */
+  isCreateSelectOptionObject?: boolean;
   /** Value to indicate if the select is modified to show that validation state.
    * If set to success, select will be modified to indicate valid state.
    * If set to error, select will be modified to indicate error state.
@@ -111,7 +117,7 @@ export interface SelectProps
   /** Callback for typeahead clear button */
   onClear?: (event: React.MouseEvent) => void;
   /** Optional callback for custom filtering */
-  onFilter?: (e: React.ChangeEvent<HTMLInputElement>, value: string) => React.ReactElement[];
+  onFilter?: (e: React.ChangeEvent<HTMLInputElement> | null, value: string) => React.ReactElement[] | undefined;
   /** Optional callback for newly created options */
   onCreateOption?: (newOptionValue: string) => void;
   /** Optional event handler called each time the value in the typeahead input changes. */
@@ -138,8 +144,10 @@ export interface SelectProps
   chipGroupProps?: Omit<ChipGroupProps, 'children' | 'ref'>;
   /** Optional props to render custom chip group in the typeaheadmulti variant */
   chipGroupComponent?: React.ReactNode;
-  /** @beta Flag for retaining keyboard-entered value in typeahead text field when focus leaves input away */
+  /** Flag for retaining keyboard-entered value in typeahead text field when focus leaves input away */
   isInputValuePersisted?: boolean;
+  /** @beta Flag for retaining filter results on blur from keyboard-entered typeahead text */
+  isInputFilterPersisted?: boolean;
   /** Content rendered in the footer of the select menu */
   footer?: React.ReactNode;
 }
@@ -154,6 +162,7 @@ export interface SelectState {
   tabbedIntoFavoritesMenu: boolean;
   typeaheadStoredIndex: number;
   ouiaStateId: string;
+  viewMoreNextIndex: number;
 }
 
 export class Select extends React.Component<SelectProps & OUIAProps, SelectState> {
@@ -170,12 +179,14 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
   static defaultProps: PickOptional<SelectProps> = {
     children: [] as React.ReactElement[],
     className: '',
+    position: SelectPosition.left,
     direction: SelectDirection.down,
     toggleId: null as string,
     isOpen: false,
     isGrouped: false,
     isPlain: false,
     isDisabled: false,
+    hasPlaceholderStyle: false,
     isCreatable: false,
     validated: 'default',
     'aria-label': '',
@@ -207,7 +218,9 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
     favoritesLabel: 'Favorites',
     ouiaSafe: true,
     chipGroupComponent: null,
-    isInputValuePersisted: false
+    isInputValuePersisted: false,
+    isInputFilterPersisted: false,
+    isCreateSelectOptionObject: false
   };
 
   state: SelectState = {
@@ -219,7 +232,8 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
     typeaheadStoredIndex: -1,
     creatableValue: '',
     tabbedIntoFavoritesMenu: false,
-    ouiaStateId: getDefaultOUIAId(Select.displayName, this.props.variant)
+    ouiaStateId: getDefaultOUIAId(Select.displayName, this.props.variant),
+    viewMoreNextIndex: -1
   };
 
   getTypeaheadActiveChild = (typeaheadCurrIndex: number) =>
@@ -230,6 +244,7 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
       this.refCollection[0][0] = this.filterRef.current;
     }
 
+    // Move focus to top of the menu if state.focusFirstOption was updated to true and the menu does not have custom content
     if (!prevState.focusFirstOption && this.state.focusFirstOption && !this.props.customContent) {
       const firstRef = this.refCollection.find(ref => ref !== null);
       if (firstRef && firstRef[0]) {
@@ -237,10 +252,32 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
       }
     }
 
-    if (prevProps.children !== this.props.children) {
+    // if viewMoreNextIndex is not -1, view more was clicked, set focus on first newly loaded item
+    if (
+      this.state.viewMoreNextIndex !== -1 &&
+      this.refCollection.length > this.state.viewMoreNextIndex &&
+      this.props.loadingVariant !== 'spinner'
+    ) {
+      this.refCollection[this.state.viewMoreNextIndex][0].focus();
+    }
+
+    // the number or contents of the children has changed, update state.typeaheadFilteredChildren
+    if (
+      prevProps.children.length !== this.props.children.length ||
+      prevProps.children.some((child: React.ReactElement, index: number) => {
+        if (child.props && this.props.children[index].props) {
+          return child.props.value !== this.props.children[index].props.value;
+        } else {
+          return child !== this.props.children[index];
+        }
+      })
+    ) {
       this.updateTypeAheadFilteredChildren(prevState.typeaheadInputValue || '', null);
     }
 
+    // for menus with favorites,
+    // if the number of favorites or typeahead filtered children has changed, the generated
+    // list of favorites needs to be updated
     if (
       this.props.onFavorite &&
       (this.props.favorites.length !== prevProps.favorites.length ||
@@ -285,10 +322,14 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
   };
 
   onClose = () => {
+    const { isInputFilterPersisted } = this.props;
+
     this.setState({
       focusFirstOption: false,
       typeaheadInputValue: null,
-      typeaheadFilteredChildren: React.Children.toArray(this.props.children),
+      ...(!isInputFilterPersisted && {
+        typeaheadFilteredChildren: React.Children.toArray(this.props.children)
+      }),
       typeaheadCurrIndex: -1,
       tabbedIntoFavoritesMenu: false
     });
@@ -315,7 +356,16 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
   updateTypeAheadFilteredChildren = (typeaheadInputValue: string, e: React.ChangeEvent<HTMLInputElement> | null) => {
     let typeaheadFilteredChildren: any;
 
-    const { onFilter, isCreatable, onCreateOption, createText, noResultsFoundText, children, isGrouped } = this.props;
+    const {
+      onFilter,
+      isCreatable,
+      onCreateOption,
+      createText,
+      noResultsFoundText,
+      children,
+      isGrouped,
+      isCreateSelectOptionObject
+    } = this.props;
 
     if (onFilter) {
       /* The updateTypeAheadFilteredChildren callback is not only called on input changes but also when the children change.
@@ -332,11 +382,13 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
       const childrenArray = React.Children.toArray(children) as React.ReactElement<SelectGroupProps>[];
       if (isGrouped) {
         const childFilter = (child: React.ReactElement<SelectGroupProps>) =>
-          child.props.value && this.getDisplay(child.props.value.toString(), 'text').search(input) === 0;
+          child.props.value &&
+          child.props.value.toString &&
+          this.getDisplay(child.props.value.toString(), 'text').search(input) === 0;
         typeaheadFilteredChildren =
           typeaheadInputValue.toString() !== ''
             ? React.Children.map(children, group => {
-                if (group.type === SelectGroup) {
+                if (React.isValidElement(group) && group.type === SelectGroup) {
                   const filteredGroupChildren = (React.Children.toArray(group.props.children) as React.ReactElement<
                     SelectGroupProps
                   >[]).filter(childFilter);
@@ -381,16 +433,37 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
     if (typeaheadFilteredChildren.length === 0) {
       !isCreatable &&
         typeaheadFilteredChildren.push(
-          <SelectOption isDisabled key={0} value={noResultsFoundText} isNoResultsOption />
+          <SelectOption isDisabled key="no-results" value={noResultsFoundText} isNoResultsOption />
         );
     }
     if (isCreatable && typeaheadInputValue !== '') {
       const newValue = typeaheadInputValue;
-      typeaheadFilteredChildren.push(
-        <SelectOption key={0} value={newValue} onClick={() => onCreateOption && onCreateOption(newValue)}>
-          {createText} "{newValue}"
-        </SelectOption>
-      );
+      if (
+        !typeaheadFilteredChildren.find(
+          (i: React.ReactElement) =>
+            i.props.value && i.props.value.toString().toLowerCase() === newValue.toString().toLowerCase()
+        )
+      ) {
+        const newOptionValue = isCreateSelectOptionObject
+          ? ({
+              toString: () => newValue,
+              compareTo: value =>
+                this.toString()
+                  .toLowerCase()
+                  .includes(value.toString().toLowerCase())
+            } as SelectOptionObject)
+          : newValue;
+
+        typeaheadFilteredChildren.push(
+          <SelectOption
+            key={`create ${newValue}`}
+            value={newOptionValue}
+            onClick={() => onCreateOption && onCreateOption(newValue)}
+          >
+            {createText} "{newValue}"
+          </SelectOption>
+        );
+      }
     }
 
     this.setState({
@@ -576,6 +649,9 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
           this.onToggle(false);
         }
       } else if (!tabbedIntoFavoritesMenu) {
+        if (this.refCollection[0][0] === null) {
+          return;
+        }
         let nextIndex;
         if (typeaheadCurrIndex === -1 && position === 'down') {
           nextIndex = 0;
@@ -653,6 +729,13 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
     return null;
   };
 
+  setVieMoreNextIndex = () => {
+    this.setState({ viewMoreNextIndex: this.refCollection.length - 1 });
+  };
+
+  isLastOptionBeforeFooter = (index: any) =>
+    this.props.footer && index === this.refCollection.length - 1 ? true : false;
+
   render() {
     const {
       children,
@@ -669,6 +752,7 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
       isGrouped,
       isPlain,
       isDisabled,
+      hasPlaceholderStyle,
       validated,
       selections: selectionsProp,
       typeAheadAriaLabel,
@@ -701,6 +785,7 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
       inputIdPrefix,
       /* eslint-disable @typescript-eslint/no-unused-vars */
       isInputValuePersisted,
+      isInputFilterPersisted,
       /* eslint-enable @typescript-eslint/no-unused-vars */
       menuAppendTo,
       favorites,
@@ -709,6 +794,7 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
       favoritesLabel,
       footer,
       loadingVariant,
+      isCreateSelectOptionObject,
       ...props
     } = this.props;
     const {
@@ -720,6 +806,11 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
     } = this.state;
     const selectToggleId = toggleId || `pf-select-toggle-id-${currentId++}`;
     const selections = Array.isArray(selectionsProp) ? selectionsProp : [selectionsProp];
+    // Find out if the selected option is a placeholder
+    const selectedOption = React.Children.toArray(children).find(
+      (option: any) => option.props.value === selections[0]
+    ) as any;
+    const isSelectedPlaceholder = selectedOption && selectedOption.props.isPlaceholder;
     const hasAnySelections = Boolean(selections[0] && selections[0] !== '');
     const typeaheadActiveChild = this.getTypeaheadActiveChild(typeaheadCurrIndex);
     let childPlaceholderText = null as string;
@@ -761,7 +852,13 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
           );
         } else if (loadingVariant?.text) {
           renderableItems.push(
-            <SelectOption isLoad key="loading" value={loadingVariant.text} onClick={loadingVariant?.onClick} />
+            <SelectOption
+              isLoad
+              key="loading"
+              value={loadingVariant.text}
+              setViewMoreNextIndex={this.setVieMoreNextIndex}
+              onClick={loadingVariant?.onClick}
+            />
           );
         }
       }
@@ -845,7 +942,7 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
                 }
               }}
               ref={this.filterRef}
-              autoComplete="off"
+              autoComplete="never"
             />
           </div>
           <Divider key="inline-filter-divider" />
@@ -922,6 +1019,7 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
         ref={this.menuComponentRef}
         footer={footer}
         footerRef={this.footerRef}
+        isLastOptionBeforeFooter={this.isLastOptionBeforeFooter}
       >
         {variantChildren}
       </SelectMenu>
@@ -972,6 +1070,9 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
           {...(footer && { footerRef: this.footerRef })}
           isOpen={isOpen}
           isPlain={isPlain}
+          hasPlaceholderStyle={
+            hasPlaceholderStyle && (!selections.length || selections[0] === null || isSelectedPlaceholder)
+          }
           onToggle={this.onToggle}
           onEnter={this.onEnter}
           onClose={this.onClose}
@@ -1035,7 +1136,7 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
                   type="text"
                   onClick={this.onClick}
                   onChange={this.onChange}
-                  autoComplete="off"
+                  autoComplete="never"
                   disabled={isDisabled}
                   ref={this.inputRef}
                 />
@@ -1059,7 +1160,7 @@ export class Select extends React.Component<SelectProps & OUIAProps, SelectState
                   type="text"
                   onChange={this.onChange}
                   onClick={this.onClick}
-                  autoComplete="off"
+                  autoComplete="never"
                   disabled={isDisabled}
                   ref={this.inputRef}
                 />
